@@ -88,7 +88,7 @@ mm_iface_modem_messaging_bind_simple_status (MMIfaceModemMessaging *self,
 
 /*****************************************************************************/
 
-MMSms *
+MMBaseSms *
 mm_iface_modem_messaging_create_sms (MMIfaceModemMessaging *self)
 {
     g_assert (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->create_sms != NULL);
@@ -271,7 +271,7 @@ handle_create_auth_ready (MMBaseModem *self,
     MMSmsList *list = NULL;
     GError *error = NULL;
     MMSmsProperties *properties;
-    MMSms *sms;
+    MMBaseSms *sms;
 
     if (!mm_base_modem_authorize_finish (self, res, &error)) {
         g_dbus_method_invocation_take_error (ctx->invocation, error);
@@ -300,9 +300,9 @@ handle_create_auth_ready (MMBaseModem *self,
         return;
     }
 
-    sms = mm_sms_new_from_properties (MM_BASE_MODEM (self),
-                                      properties,
-                                      &error);
+    sms = mm_base_sms_new_from_properties (MM_BASE_MODEM (self),
+                                           properties,
+                                           &error);
     if (!sms) {
         g_object_unref (properties);
         g_dbus_method_invocation_take_error (ctx->invocation, error);
@@ -330,7 +330,7 @@ handle_create_auth_ready (MMBaseModem *self,
     /* Complete the DBus call */
     mm_gdbus_modem_messaging_complete_create (ctx->skeleton,
                                               ctx->invocation,
-                                              mm_sms_get_path (sms));
+                                              mm_base_sms_get_path (sms));
     g_object_unref (sms);
 
     g_object_unref (properties);
@@ -489,6 +489,17 @@ mm_iface_modem_messaging_is_storage_supported_for_receiving (MMIfaceModemMessagi
 
 /*****************************************************************************/
 
+static void
+update_message_list (MmGdbusModemMessaging *skeleton,
+                     MMSmsList *list)
+{
+    gchar **paths;
+
+    paths = mm_sms_list_get_paths (list);
+    mm_gdbus_modem_messaging_set_messages (skeleton, (const gchar *const *)paths);
+    g_strfreev (paths);
+}
+
 static void
 sms_added (MMSmsList *list,
            const gchar *sms_path,
@@ -498,6 +509,7 @@ sms_added (MMSmsList *list,
     mm_dbg ("Added %s SMS at '%s'",
             received ? "received" : "local",
             sms_path);
+    update_message_list (skeleton, list);
     mm_gdbus_modem_messaging_emit_added (skeleton, sms_path, received);
 }
 
@@ -507,13 +519,14 @@ sms_deleted (MMSmsList *list,
              MmGdbusModemMessaging *skeleton)
 {
     mm_dbg ("Deleted SMS at '%s'", sms_path);
+    update_message_list (skeleton, list);
     mm_gdbus_modem_messaging_emit_deleted (skeleton, sms_path);
 }
 
 /*****************************************************************************/
 
 typedef struct _DisablingContext DisablingContext;
-static void interface_disabling_step (DisablingContext *ctx);
+static void interface_disabling_step (GTask *task);
 
 typedef enum {
     DISABLING_STEP_FIRST,
@@ -523,18 +536,13 @@ typedef enum {
 } DisablingStep;
 
 struct _DisablingContext {
-    MMIfaceModemMessaging *self;
     DisablingStep step;
-    GSimpleAsyncResult *result;
     MmGdbusModemMessaging *skeleton;
 };
 
 static void
-disabling_context_complete_and_free (DisablingContext *ctx)
+disabling_context_free (DisablingContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->self);
-    g_object_unref (ctx->result);
     if (ctx->skeleton)
         g_object_unref (ctx->skeleton);
     g_free (ctx);
@@ -545,50 +553,60 @@ mm_iface_modem_messaging_disable_finish (MMIfaceModemMessaging *self,
                                         GAsyncResult *res,
                                         GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 disable_unsolicited_events_ready (MMIfaceModemMessaging *self,
                                   GAsyncResult *res,
-                                  DisablingContext *ctx)
+                                  GTask *task)
 {
+    DisablingContext *ctx;
     GError *error = NULL;
 
     MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->disable_unsolicited_events_finish (self, res, &error);
     if (error) {
-        g_simple_async_result_take_error (ctx->result, error);
-        disabling_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_disabling_step (ctx);
+    interface_disabling_step (task);
 }
 
 static void
 cleanup_unsolicited_events_ready (MMIfaceModemMessaging *self,
                                   GAsyncResult *res,
-                                  DisablingContext *ctx)
+                                  GTask *task)
 {
+    DisablingContext *ctx;
     GError *error = NULL;
 
     MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->cleanup_unsolicited_events_finish (self, res, &error);
     if (error) {
-        g_simple_async_result_take_error (ctx->result, error);
-        disabling_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_disabling_step (ctx);
+    interface_disabling_step (task);
 }
 
 static void
-interface_disabling_step (DisablingContext *ctx)
+interface_disabling_step (GTask *task)
 {
+    MMIfaceModemMessaging *self;
+    DisablingContext *ctx;
+
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
+
     switch (ctx->step) {
     case DISABLING_STEP_FIRST:
         /* Fall down to next step */
@@ -596,12 +614,12 @@ interface_disabling_step (DisablingContext *ctx)
 
     case DISABLING_STEP_DISABLE_UNSOLICITED_EVENTS:
         /* Allow cleaning up unsolicited events */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->disable_unsolicited_events &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->disable_unsolicited_events_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->disable_unsolicited_events (
-                ctx->self,
+        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->disable_unsolicited_events &&
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->disable_unsolicited_events_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->disable_unsolicited_events (
+                self,
                 (GAsyncReadyCallback)disable_unsolicited_events_ready,
-                ctx);
+                task);
             return;
         }
         /* Fall down to next step */
@@ -609,12 +627,12 @@ interface_disabling_step (DisablingContext *ctx)
 
     case DISABLING_STEP_CLEANUP_UNSOLICITED_EVENTS:
         /* Allow cleaning up unsolicited events */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->cleanup_unsolicited_events &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->cleanup_unsolicited_events_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->cleanup_unsolicited_events (
-                ctx->self,
+        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->cleanup_unsolicited_events &&
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->cleanup_unsolicited_events_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->cleanup_unsolicited_events (
+                self,
                 (GAsyncReadyCallback)cleanup_unsolicited_events_ready,
-                ctx);
+                task);
             return;
         }
         /* Fall down to next step */
@@ -622,13 +640,13 @@ interface_disabling_step (DisablingContext *ctx)
 
     case DISABLING_STEP_LAST:
         /* Clear SMS list */
-        g_object_set (ctx->self,
+        g_object_set (self,
                       MM_IFACE_MODEM_MESSAGING_SMS_LIST, NULL,
                       NULL);
 
         /* We are done without errors! */
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        disabling_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
@@ -641,33 +659,33 @@ mm_iface_modem_messaging_disable (MMIfaceModemMessaging *self,
                                   gpointer user_data)
 {
     DisablingContext *ctx;
+    GTask *task;
 
     ctx = g_new0 (DisablingContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_iface_modem_messaging_disable);
     ctx->step = DISABLING_STEP_FIRST;
-    g_object_get (ctx->self,
+
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)disabling_context_free);
+
+    g_object_get (self,
                   MM_IFACE_MODEM_MESSAGING_DBUS_SKELETON, &ctx->skeleton,
                   NULL);
     if (!ctx->skeleton) {
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Couldn't get interface skeleton");
-        disabling_context_complete_and_free (ctx);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Couldn't get interface skeleton");
+        g_object_unref (task);
         return;
     }
 
-    interface_disabling_step (ctx);
+    interface_disabling_step (task);
 }
 
 /*****************************************************************************/
 
 typedef struct _EnablingContext EnablingContext;
-static void interface_enabling_step (EnablingContext *ctx);
+static void interface_enabling_step (GTask *task);
 
 typedef enum {
     ENABLING_STEP_FIRST,
@@ -680,81 +698,65 @@ typedef enum {
 } EnablingStep;
 
 struct _EnablingContext {
-    MMIfaceModemMessaging *self;
     EnablingStep step;
-    GSimpleAsyncResult *result;
-    GCancellable *cancellable;
     MmGdbusModemMessaging *skeleton;
     guint mem1_storage_index;
 };
 
 static void
-enabling_context_complete_and_free (EnablingContext *ctx)
+enabling_context_free (EnablingContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->self);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->cancellable);
     if (ctx->skeleton)
         g_object_unref (ctx->skeleton);
     g_free (ctx);
 }
 
-static gboolean
-enabling_context_complete_and_free_if_cancelled (EnablingContext *ctx)
-{
-    if (!g_cancellable_is_cancelled (ctx->cancellable))
-        return FALSE;
-
-    g_simple_async_result_set_error (ctx->result,
-                                     MM_CORE_ERROR,
-                                     MM_CORE_ERROR_CANCELLED,
-                                     "Interface enabling cancelled");
-    enabling_context_complete_and_free (ctx);
-    return TRUE;
-}
-
 gboolean
 mm_iface_modem_messaging_enable_finish (MMIfaceModemMessaging *self,
                                         GAsyncResult *res,
                                         GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 setup_sms_format_ready (MMIfaceModemMessaging *self,
                         GAsyncResult *res,
-                        EnablingContext *ctx)
+                        GTask *task)
 {
+    EnablingContext *ctx;
     GError *error = NULL;
 
     MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_sms_format_finish (self, res, &error);
     if (error) {
-        g_simple_async_result_take_error (ctx->result, error);
-        enabling_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_enabling_step (ctx);
+    interface_enabling_step (task);
 }
 
-static void load_initial_sms_parts_from_storages (EnablingContext *ctx);
+static void load_initial_sms_parts_from_storages (GTask *task);
 
 static void
 load_initial_sms_parts_ready (MMIfaceModemMessaging *self,
                               GAsyncResult *res,
-                              EnablingContext *ctx)
+                              GTask *task)
 {
+    EnablingContext *ctx;
     GError *error = NULL;
 
+    ctx = g_task_get_task_data (task);
+
     MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_initial_sms_parts_finish (self, res, &error);
     if (error) {
         StorageContext *storage_ctx;
 
-        storage_ctx = get_storage_context (ctx->self);
+        storage_ctx = get_storage_context (self);
         mm_dbg ("Couldn't load SMS parts from storage '%s': '%s'",
                 mm_sms_storage_get_string (g_array_index (storage_ctx->supported_mem1,
                                                           MMSmsStorage,
@@ -765,40 +767,48 @@ load_initial_sms_parts_ready (MMIfaceModemMessaging *self,
 
     /* Go on with the storage iteration */
     ctx->mem1_storage_index++;
-    load_initial_sms_parts_from_storages (ctx);
+    load_initial_sms_parts_from_storages (task);
 }
 
 static void
 set_default_storage_ready (MMIfaceModemMessaging *self,
                            GAsyncResult *res,
-                           EnablingContext *ctx)
+                           GTask *task)
 {
+    EnablingContext *ctx;
     GError *error = NULL;
 
     if (!MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->set_default_storage_finish (self, res, &error)) {
-        mm_dbg ("Couldn't set default storage: '%s'", error->message);
+        mm_warn ("Could not set default storage: '%s'", error->message);
         g_error_free (error);
     }
 
     /* Go on with next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_enabling_step (ctx);
+    interface_enabling_step (task);
 }
 
 static void
-load_initial_sms_parts_from_storages (EnablingContext *ctx)
+load_initial_sms_parts_from_storages (GTask *task)
 {
+    MMIfaceModemMessaging *self;
+    EnablingContext *ctx;
     gboolean all_loaded = FALSE;
     StorageContext *storage_ctx;
 
-    storage_ctx = get_storage_context (ctx->self);
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
+    storage_ctx = get_storage_context (self);
 
     if (!storage_ctx->supported_mem1 || ctx->mem1_storage_index >= storage_ctx->supported_mem1->len)
         all_loaded = TRUE;
-    /* We'll skip the 'MT' storage, as that is a combination of 'SM' and 'ME' */
-    else if (g_array_index (storage_ctx->supported_mem1,
-                            MMSmsStorage,
-                            ctx->mem1_storage_index) == MM_SMS_STORAGE_MT) {
+    /* We'll skip the 'MT' storage, as that is a combination of 'SM' and 'ME'; but only if
+     * this is not the only one in the list. */
+    else if ((g_array_index (storage_ctx->supported_mem1,
+                             MMSmsStorage,
+                             ctx->mem1_storage_index) == MM_SMS_STORAGE_MT) &&
+             (storage_ctx->supported_mem1->len > 1)) {
         ctx->mem1_storage_index++;
         if (ctx->mem1_storage_index >= storage_ctx->supported_mem1->len)
             all_loaded = TRUE;
@@ -807,44 +817,46 @@ load_initial_sms_parts_from_storages (EnablingContext *ctx)
     if (all_loaded) {
         /* Go on with next step */
         ctx->step++;
-        interface_enabling_step (ctx);
+        interface_enabling_step (task);
         return;
     }
 
-    MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->load_initial_sms_parts (
-        ctx->self,
+    MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_initial_sms_parts (
+        self,
         g_array_index (storage_ctx->supported_mem1,
                        MMSmsStorage,
                        ctx->mem1_storage_index),
         (GAsyncReadyCallback)load_initial_sms_parts_ready,
-        ctx);
-    return;
+        task);
 }
 
 static void
 setup_unsolicited_events_ready (MMIfaceModemMessaging *self,
                                 GAsyncResult *res,
-                                EnablingContext *ctx)
+                                GTask *task)
 {
+    EnablingContext *ctx;
     GError *error = NULL;
 
     MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_unsolicited_events_finish (self, res, &error);
     if (error) {
-        g_simple_async_result_take_error (ctx->result, error);
-        enabling_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_enabling_step (ctx);
+    interface_enabling_step (task);
 }
 
 static void
 enable_unsolicited_events_ready (MMIfaceModemMessaging *self,
                                  GAsyncResult *res,
-                                 EnablingContext *ctx)
+                                 GTask *task)
 {
+    EnablingContext *ctx;
     GError *error = NULL;
 
     /* Not critical! */
@@ -854,8 +866,9 @@ enable_unsolicited_events_ready (MMIfaceModemMessaging *self,
     }
 
     /* Go on with next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_enabling_step (ctx);
+    interface_enabling_step (task);
 }
 
 static MMSmsStorage
@@ -884,18 +897,26 @@ get_best_initial_default_sms_storage (MMIfaceModemMessaging *self)
 }
 
 static void
-interface_enabling_step (EnablingContext *ctx)
+interface_enabling_step (GTask *task)
 {
+    MMIfaceModemMessaging *self;
+    EnablingContext *ctx;
+
     /* Don't run new steps if we're cancelled */
-    if (enabling_context_complete_and_free_if_cancelled (ctx))
+    if (g_task_return_error_if_cancelled (task)) {
+        g_object_unref (task);
         return;
+    }
+
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
 
     switch (ctx->step) {
     case ENABLING_STEP_FIRST: {
         MMSmsList *list;
 
-        list = mm_sms_list_new (MM_BASE_MODEM (ctx->self));
-        g_object_set (ctx->self,
+        list = mm_sms_list_new (MM_BASE_MODEM (self));
+        g_object_set (self,
                       MM_IFACE_MODEM_MESSAGING_SMS_LIST, list,
                       NULL);
 
@@ -917,12 +938,12 @@ interface_enabling_step (EnablingContext *ctx)
 
     case ENABLING_STEP_SETUP_SMS_FORMAT:
         /* Allow setting SMS format to use */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->setup_sms_format &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->setup_sms_format_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->setup_sms_format (
-                ctx->self,
+        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_sms_format &&
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_sms_format_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_sms_format (
+                self,
                 (GAsyncReadyCallback)setup_sms_format_ready,
-                ctx);
+                task);
             return;
         }
         /* Fall down to next step */
@@ -930,23 +951,23 @@ interface_enabling_step (EnablingContext *ctx)
 
     case ENABLING_STEP_STORAGE_DEFAULTS:
         /* Set storage defaults */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->set_default_storage &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->set_default_storage_finish) {
+        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->set_default_storage &&
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->set_default_storage_finish) {
             MMSmsStorage default_storage;
 
-            default_storage = get_best_initial_default_sms_storage (ctx->self);
+            default_storage = get_best_initial_default_sms_storage (self);
 
             /* Already bound to the 'default-storage' property in the skeleton */
-            g_object_set (ctx->self,
+            g_object_set (self,
                           MM_IFACE_MODEM_MESSAGING_SMS_DEFAULT_STORAGE, default_storage,
                           NULL);
 
             if (default_storage != MM_SMS_STORAGE_UNKNOWN) {
-                MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->set_default_storage (
-                    ctx->self,
+                MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->set_default_storage (
+                    self,
                     default_storage,
                     (GAsyncReadyCallback)set_default_storage_ready,
-                    ctx);
+                    task);
                 return;
             }
 
@@ -957,9 +978,9 @@ interface_enabling_step (EnablingContext *ctx)
 
     case ENABLING_STEP_LOAD_INITIAL_SMS_PARTS:
         /* Allow loading the initial list of SMS parts */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->load_initial_sms_parts &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->load_initial_sms_parts_finish) {
-            load_initial_sms_parts_from_storages (ctx);
+        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_initial_sms_parts &&
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_initial_sms_parts_finish) {
+            load_initial_sms_parts_from_storages (task);
             return;
         }
         /* Fall down to next step */
@@ -967,12 +988,12 @@ interface_enabling_step (EnablingContext *ctx)
 
     case ENABLING_STEP_SETUP_UNSOLICITED_EVENTS:
         /* Allow setting up unsolicited events */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->setup_unsolicited_events &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->setup_unsolicited_events_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->setup_unsolicited_events (
-                ctx->self,
+        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_unsolicited_events &&
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_unsolicited_events_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->setup_unsolicited_events (
+                self,
                 (GAsyncReadyCallback)setup_unsolicited_events_ready,
-                ctx);
+                task);
             return;
         }
         /* Fall down to next step */
@@ -980,12 +1001,12 @@ interface_enabling_step (EnablingContext *ctx)
 
     case ENABLING_STEP_ENABLE_UNSOLICITED_EVENTS:
         /* Allow setting up unsolicited events */
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->enable_unsolicited_events &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->enable_unsolicited_events_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->enable_unsolicited_events (
-                ctx->self,
+        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->enable_unsolicited_events &&
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->enable_unsolicited_events_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->enable_unsolicited_events (
+                self,
                 (GAsyncReadyCallback)enable_unsolicited_events_ready,
-                ctx);
+                task);
             return;
         }
         /* Fall down to next step */
@@ -993,8 +1014,8 @@ interface_enabling_step (EnablingContext *ctx)
 
     case ENABLING_STEP_LAST:
         /* We are done without errors! */
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        enabling_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
@@ -1008,84 +1029,80 @@ mm_iface_modem_messaging_enable (MMIfaceModemMessaging *self,
                                  gpointer user_data)
 {
     EnablingContext *ctx;
+    GTask *task;
 
     ctx = g_new0 (EnablingContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->cancellable = g_object_ref (cancellable);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_iface_modem_messaging_enable);
     ctx->step = ENABLING_STEP_FIRST;
-    g_object_get (ctx->self,
+
+    task = g_task_new (self, cancellable, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)enabling_context_free);
+
+    g_object_get (self,
                   MM_IFACE_MODEM_MESSAGING_DBUS_SKELETON, &ctx->skeleton,
                   NULL);
     if (!ctx->skeleton) {
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Couldn't get interface skeleton");
-        enabling_context_complete_and_free (ctx);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Couldn't get interface skeleton");
+        g_object_unref (task);
         return;
     }
 
-    interface_enabling_step (ctx);
+    interface_enabling_step (task);
 }
 
 /*****************************************************************************/
 
 typedef struct _InitializationContext InitializationContext;
-static void interface_initialization_step (InitializationContext *ctx);
+static void interface_initialization_step (GTask *task);
 
 typedef enum {
     INITIALIZATION_STEP_FIRST,
     INITIALIZATION_STEP_CHECK_SUPPORT,
     INITIALIZATION_STEP_FAIL_IF_UNSUPPORTED,
     INITIALIZATION_STEP_LOAD_SUPPORTED_STORAGES,
+    INITIALIZATION_STEP_INIT_CURRENT_STORAGES,
     INITIALIZATION_STEP_LAST
 } InitializationStep;
 
 struct _InitializationContext {
-    MMIfaceModemMessaging *self;
     MmGdbusModemMessaging *skeleton;
-    GCancellable *cancellable;
-    GSimpleAsyncResult *result;
     InitializationStep step;
 };
 
 static void
-initialization_context_complete_and_free (InitializationContext *ctx)
+initialization_context_free (InitializationContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->self);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->cancellable);
     g_object_unref (ctx->skeleton);
     g_free (ctx);
 }
 
-static gboolean
-initialization_context_complete_and_free_if_cancelled (InitializationContext *ctx)
+static void
+skip_unknown_storages (GArray *mem)
 {
-    if (!g_cancellable_is_cancelled (ctx->cancellable))
-        return FALSE;
+    guint i = mem->len;
 
-    g_simple_async_result_set_error (ctx->result,
-                                     MM_CORE_ERROR,
-                                     MM_CORE_ERROR_CANCELLED,
-                                     "Interface initialization cancelled");
-    initialization_context_complete_and_free (ctx);
-    return TRUE;
+    if (!mem)
+        return;
+
+    /* Remove UNKNOWN from the list of supported storages */
+    while (i-- > 0) {
+        if (g_array_index (mem, MMSmsStorage, i) == MM_SMS_STORAGE_UNKNOWN)
+            g_array_remove_index (mem, i);
+    }
 }
 
 static void
 load_supported_storages_ready (MMIfaceModemMessaging *self,
                                GAsyncResult *res,
-                               InitializationContext *ctx)
+                               GTask *task)
 {
+    InitializationContext *ctx;
     StorageContext *storage_ctx;
     GError *error = NULL;
 
+    ctx = g_task_get_task_data (task);
     storage_ctx = get_storage_context (self);
     if (!MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_supported_storages_finish (
             self,
@@ -1103,6 +1120,11 @@ load_supported_storages_ready (MMIfaceModemMessaging *self,
         GArray *supported_storages;
         guint i;
 
+        /* Never add unknown storages */
+        skip_unknown_storages (storage_ctx->supported_mem1);
+        skip_unknown_storages (storage_ctx->supported_mem2);
+        skip_unknown_storages (storage_ctx->supported_mem3);
+
         mem1 = mm_common_build_sms_storages_string ((MMSmsStorage *)storage_ctx->supported_mem1->data,
                                                     storage_ctx->supported_mem1->len);
         mem2 = mm_common_build_sms_storages_string ((MMSmsStorage *)storage_ctx->supported_mem2->data,
@@ -1147,14 +1169,15 @@ load_supported_storages_ready (MMIfaceModemMessaging *self,
 
     /* Go on to next step */
     ctx->step++;
-    interface_initialization_step (ctx);
+    interface_initialization_step (task);
 }
 
 static void
 check_support_ready (MMIfaceModemMessaging *self,
                      GAsyncResult *res,
-                     InitializationContext *ctx)
+                     GTask *task)
 {
+    InitializationContext *ctx;
     GError *error = NULL;
 
     if (!MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->check_support_finish (self,
@@ -1173,16 +1196,51 @@ check_support_ready (MMIfaceModemMessaging *self,
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_initialization_step (ctx);
+    interface_initialization_step (task);
 }
 
 static void
-interface_initialization_step (InitializationContext *ctx)
+init_current_storages_ready (MMIfaceModemMessaging *self,
+                             GAsyncResult *res,
+                             GTask *task)
 {
+    InitializationContext *ctx;
+    StorageContext *storage_ctx;
+    GError *error = NULL;
+
+    storage_ctx = get_storage_context (self);
+    if (!MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->init_current_storages_finish (
+            self,
+            res,
+            &error)) {
+        mm_dbg ("Couldn't initialize current storages: '%s'", error->message);
+        g_error_free (error);
+    } else {
+        mm_dbg ("Current storages initialized");
+    }
+
+    /* Go on to next step */
+    ctx = g_task_get_task_data (task);
+    ctx->step++;
+    interface_initialization_step (task);
+}
+
+static void
+interface_initialization_step (GTask *task)
+{
+    MMIfaceModemMessaging *self;
+    InitializationContext *ctx;
+
     /* Don't run new steps if we're cancelled */
-    if (initialization_context_complete_and_free_if_cancelled (ctx))
+    if (g_task_return_error_if_cancelled (task)) {
+        g_object_unref (task);
         return;
+    }
+
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
 
     switch (ctx->step) {
     case INITIALIZATION_STEP_FIRST:
@@ -1198,23 +1256,23 @@ interface_initialization_step (InitializationContext *ctx)
         ctx->step++;
 
     case INITIALIZATION_STEP_CHECK_SUPPORT:
-        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (ctx->self),
+        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (self),
                                                    support_checked_quark))) {
             /* Set the checked flag so that we don't run it again */
-            g_object_set_qdata (G_OBJECT (ctx->self),
+            g_object_set_qdata (G_OBJECT (self),
                                 support_checked_quark,
                                 GUINT_TO_POINTER (TRUE));
             /* Initially, assume we don't support it */
-            g_object_set_qdata (G_OBJECT (ctx->self),
+            g_object_set_qdata (G_OBJECT (self),
                                 supported_quark,
                                 GUINT_TO_POINTER (FALSE));
 
-            if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->check_support &&
-                MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->check_support_finish) {
-                MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->check_support (
-                    ctx->self,
+            if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->check_support &&
+                MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->check_support_finish) {
+                MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->check_support (
+                    self,
                     (GAsyncReadyCallback)check_support_ready,
-                    ctx);
+                    task);
                 return;
             }
 
@@ -1225,25 +1283,37 @@ interface_initialization_step (InitializationContext *ctx)
         ctx->step++;
 
     case INITIALIZATION_STEP_FAIL_IF_UNSUPPORTED:
-        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (ctx->self),
+        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (self),
                                                    supported_quark))) {
-            g_simple_async_result_set_error (ctx->result,
-                                             MM_CORE_ERROR,
-                                             MM_CORE_ERROR_UNSUPPORTED,
-                                             "Messaging not supported");
-            initialization_context_complete_and_free (ctx);
+            g_task_return_new_error (task,
+                                     MM_CORE_ERROR,
+                                     MM_CORE_ERROR_UNSUPPORTED,
+                                     "Messaging not supported");
+            g_object_unref (task);
             return;
         }
         /* Fall down to next step */
         ctx->step++;
 
     case INITIALIZATION_STEP_LOAD_SUPPORTED_STORAGES:
-        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->load_supported_storages &&
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->load_supported_storages_finish) {
-            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (ctx->self)->load_supported_storages (
-                ctx->self,
+        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_supported_storages &&
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_supported_storages_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->load_supported_storages (
+                self,
                 (GAsyncReadyCallback)load_supported_storages_ready,
-                ctx);
+                task);
+            return;
+        }
+        /* Fall down to next step */
+        ctx->step++;
+
+    case INITIALIZATION_STEP_INIT_CURRENT_STORAGES:
+        if (MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->init_current_storages &&
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->init_current_storages_finish) {
+            MM_IFACE_MODEM_MESSAGING_GET_INTERFACE (self)->init_current_storages (
+                self,
+                (GAsyncReadyCallback)init_current_storages_ready,
+                task);
             return;
         }
         /* Fall down to next step */
@@ -1256,22 +1326,22 @@ interface_initialization_step (InitializationContext *ctx)
         g_signal_connect (ctx->skeleton,
                           "handle-create",
                           G_CALLBACK (handle_create),
-                          ctx->self);
+                          self);
         g_signal_connect (ctx->skeleton,
                           "handle-delete",
                           G_CALLBACK (handle_delete),
-                          ctx->self);
+                          self);
         g_signal_connect (ctx->skeleton,
                           "handle-list",
                           G_CALLBACK (handle_list),
-                          ctx->self);
+                          self);
 
         /* Finally, export the new interface */
-        mm_gdbus_object_skeleton_set_modem_messaging (MM_GDBUS_OBJECT_SKELETON (ctx->self),
+        mm_gdbus_object_skeleton_set_modem_messaging (MM_GDBUS_OBJECT_SKELETON (self),
                                                       MM_GDBUS_MODEM_MESSAGING (ctx->skeleton));
 
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        initialization_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
@@ -1283,7 +1353,7 @@ mm_iface_modem_messaging_initialize_finish (MMIfaceModemMessaging *self,
                                             GAsyncResult *res,
                                             GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 void
@@ -1294,6 +1364,7 @@ mm_iface_modem_messaging_initialize (MMIfaceModemMessaging *self,
 {
     InitializationContext *ctx;
     MmGdbusModemMessaging *skeleton = NULL;
+    GTask *task;
 
     /* Did we already create it? */
     g_object_get (self,
@@ -1316,16 +1387,13 @@ mm_iface_modem_messaging_initialize (MMIfaceModemMessaging *self,
     /* Perform async initialization here */
 
     ctx = g_new0 (InitializationContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->cancellable = g_object_ref (cancellable);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_iface_modem_messaging_initialize);
     ctx->step = INITIALIZATION_STEP_FIRST;
     ctx->skeleton = skeleton;
 
-    interface_initialization_step (ctx);
+    task = g_task_new (self, cancellable, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)initialization_context_free);
+
+    interface_initialization_step (task);
 }
 
 void
