@@ -23,19 +23,21 @@
 #include <libmm-glib.h>
 
 #include "mm-charsets.h"
-#include "mm-at-serial-port.h"
-#include "mm-bearer.h"
-#include "mm-sim.h"
+#include "mm-port-serial-at.h"
+#include "mm-base-bearer.h"
+#include "mm-base-sim.h"
 
 #define MM_TYPE_IFACE_MODEM            (mm_iface_modem_get_type ())
 #define MM_IFACE_MODEM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), MM_TYPE_IFACE_MODEM, MMIfaceModem))
 #define MM_IS_IFACE_MODEM(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MM_TYPE_IFACE_MODEM))
 #define MM_IFACE_MODEM_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), MM_TYPE_IFACE_MODEM, MMIfaceModem))
 
-#define MM_IFACE_MODEM_DBUS_SKELETON "iface-modem-dbus-skeleton"
-#define MM_IFACE_MODEM_STATE         "iface-modem-state"
-#define MM_IFACE_MODEM_SIM           "iface-modem-sim"
-#define MM_IFACE_MODEM_BEARER_LIST   "iface-modem-bearer-list"
+#define MM_IFACE_MODEM_DBUS_SKELETON           "iface-modem-dbus-skeleton"
+#define MM_IFACE_MODEM_STATE                   "iface-modem-state"
+#define MM_IFACE_MODEM_SIM                     "iface-modem-sim"
+#define MM_IFACE_MODEM_BEARER_LIST             "iface-modem-bearer-list"
+#define MM_IFACE_MODEM_SIM_HOT_SWAP_SUPPORTED  "iface-modem-sim-hot-swap-supported"
+#define MM_IFACE_MODEM_SIM_HOT_SWAP_CONFIGURED "iface-modem-sim-hot-swap-configured"
 
 typedef struct _MMIfaceModem MMIfaceModem;
 
@@ -82,6 +84,14 @@ struct _MMIfaceModem {
                                      GAsyncResult *res,
                                      GError **error);
 
+    /* Loading of the HardwareRevision property */
+    void (*load_hardware_revision) (MMIfaceModem *self,
+                                    GAsyncReadyCallback callback,
+                                    gpointer user_data);
+    gchar * (*load_hardware_revision_finish) (MMIfaceModem *self,
+                                              GAsyncResult *res,
+                                              GError **error);
+
     /* Loading of the EquipmentIdentifier property */
     void (*load_equipment_identifier) (MMIfaceModem *self,
                                        GAsyncReadyCallback callback,
@@ -271,6 +281,16 @@ struct _MMIfaceModem {
                                              GAsyncResult *res,
                                              GError **error);
 
+    /* Asynchronous check to see if the SIM was swapped.
+     * Useful for when the modem changes power states since we might
+     * not get the relevant notifications from the modem. */
+    void (*check_for_sim_swap) (MMIfaceModem *self,
+                                GAsyncReadyCallback callback,
+                                gpointer user_data);
+    gboolean (*check_for_sim_swap_finish) (MMIfaceModem *self,
+                                           GAsyncResult *res,
+                                           GError **error);
+
     /* Asynchronous flow control setup */
     void (*setup_flow_control) (MMIfaceModem *self,
                                 GAsyncReadyCallback callback,
@@ -304,22 +324,37 @@ struct _MMIfaceModem {
                                          GAsyncResult *res,
                                          GError **error);
 
+    /* Asynchronous modem power-off operation */
+    void (*modem_power_off) (MMIfaceModem *self,
+                             GAsyncReadyCallback callback,
+                             gpointer user_data);
+    gboolean (*modem_power_off_finish) (MMIfaceModem *self,
+                                        GAsyncResult *res,
+                                        GError **error);
     /* Create SIM */
     void (*create_sim) (MMIfaceModem *self,
                         GAsyncReadyCallback callback,
                         gpointer user_data);
-    MMSim * (*create_sim_finish) (MMIfaceModem *self,
-                                  GAsyncResult *res,
-                                  GError **error);
+    MMBaseSim * (*create_sim_finish) (MMIfaceModem *self,
+                                      GAsyncResult *res,
+                                      GError **error);
 
     /* Create bearer */
     void (*create_bearer) (MMIfaceModem *self,
                            MMBearerProperties *properties,
                            GAsyncReadyCallback callback,
                            gpointer user_data);
-    MMBearer * (*create_bearer_finish) (MMIfaceModem *self,
-                                        GAsyncResult *res,
-                                        GError **error);
+    MMBaseBearer * (*create_bearer_finish) (MMIfaceModem *self,
+                                            GAsyncResult *res,
+                                            GError **error);
+    /* Setup SIM hot swap */
+    void (*setup_sim_hot_swap) (MMIfaceModem *self,
+                                GAsyncReadyCallback callback,
+                                gpointer user_data);
+
+    gboolean (*setup_sim_hot_swap_finish) (MMIfaceModem *self,
+                                            GAsyncResult *res,
+                                            GError **error);
 };
 
 GType mm_iface_modem_get_type (void);
@@ -397,6 +432,11 @@ MMModemLock mm_iface_modem_update_lock_info_finish (MMIfaceModem *self,
                                                     GAsyncResult *res,
                                                     GError **error);
 
+MMUnlockRetries *mm_iface_modem_get_unlock_retries (MMIfaceModem *self);
+
+void mm_iface_modem_update_unlock_retries (MMIfaceModem *self,
+                                           MMUnlockRetries *unlock_retries);
+
 /* Request signal quality check update.
  * It will not only return the signal quality status, but also set the property
  * values in the DBus interface. */
@@ -432,6 +472,9 @@ void mm_iface_modem_update_access_technologies (MMIfaceModem *self,
 void mm_iface_modem_update_signal_quality (MMIfaceModem *self,
                                            guint signal_quality);
 
+/* Allow requesting to refresh signal via polling */
+void mm_iface_modem_refresh_signal (MMIfaceModem *self);
+
 /* Allow setting allowed modes */
 void     mm_iface_modem_set_current_modes        (MMIfaceModem *self,
                                                   MMModemMode allowed,
@@ -452,13 +495,13 @@ gboolean mm_iface_modem_set_current_bands_finish (MMIfaceModem *self,
                                                   GError **error);
 
 /* Allow creating bearers */
-void     mm_iface_modem_create_bearer         (MMIfaceModem *self,
-                                               MMBearerProperties *properties,
-                                               GAsyncReadyCallback callback,
-                                               gpointer user_data);
-MMBearer *mm_iface_modem_create_bearer_finish (MMIfaceModem *self,
-                                               GAsyncResult *res,
-                                               GError **error);
+void          mm_iface_modem_create_bearer         (MMIfaceModem *self,
+                                                    MMBearerProperties *properties,
+                                                    GAsyncReadyCallback callback,
+                                                    gpointer user_data);
+MMBaseBearer *mm_iface_modem_create_bearer_finish  (MMIfaceModem *self,
+                                                    GAsyncResult *res,
+                                                    GError **error);
 
 /* Helper method to wait for a final state */
 void         mm_iface_modem_wait_for_final_state        (MMIfaceModem *self,
