@@ -21,11 +21,11 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <gio/gio.h>
-#include <gudev/gudev.h>
 
 #include "mm-private-boxed-types.h"
 #include "mm-port-probe-at.h"
-#include "mm-at-serial-port.h"
+#include "mm-port-serial-at.h"
+#include "mm-kernel-device.h"
 #include "mm-device.h"
 
 #define MM_TYPE_PORT_PROBE            (mm_port_probe_get_type ())
@@ -68,7 +68,7 @@ struct _MMPortProbeClass {
  * It also helps to implement plugin-specific checks, as plugins can set
  * their own probing results on the 'probe' object. */
 typedef void     (* MMPortProbeAtCustomInit)       (MMPortProbe *probe,
-                                                    MMAtSerialPort *port,
+                                                    MMPortSerialAt *port,
                                                     GCancellable *cancellable,
                                                     GAsyncReadyCallback callback,
                                                     gpointer user_data);
@@ -78,15 +78,15 @@ typedef gboolean (* MMPortProbeAtCustomInitFinish) (MMPortProbe *probe,
 
 GType mm_port_probe_get_type (void);
 
-MMPortProbe *mm_port_probe_new (MMDevice *device,
-                                GUdevDevice *port);
+MMPortProbe *mm_port_probe_new (MMDevice       *device,
+                                MMKernelDevice *port);
 
-MMDevice    *mm_port_probe_peek_device      (MMPortProbe *self);
-MMDevice    *mm_port_probe_get_device       (MMPortProbe *self);
-GUdevDevice *mm_port_probe_peek_port        (MMPortProbe *self);
-GUdevDevice *mm_port_probe_get_port         (MMPortProbe *self);
-const gchar *mm_port_probe_get_port_name    (MMPortProbe *self);
-const gchar *mm_port_probe_get_port_subsys  (MMPortProbe *self);
+MMDevice       *mm_port_probe_peek_device     (MMPortProbe *self);
+MMDevice       *mm_port_probe_get_device      (MMPortProbe *self);
+MMKernelDevice *mm_port_probe_peek_port       (MMPortProbe *self);
+MMKernelDevice *mm_port_probe_get_port        (MMPortProbe *self);
+const gchar    *mm_port_probe_get_port_name   (MMPortProbe *self);
+const gchar    *mm_port_probe_get_port_subsys (MMPortProbe *self);
 
 /* Probing result setters */
 void mm_port_probe_set_result_at         (MMPortProbe *self,
@@ -112,12 +112,12 @@ void     mm_port_probe_run        (MMPortProbe *self,
                                    gboolean at_send_lf,
                                    const MMPortProbeAtCommand *at_custom_probe,
                                    const MMAsyncMethod *at_custom_init,
+                                   GCancellable *cancellable,
                                    GAsyncReadyCallback callback,
                                    gpointer user_data);
 gboolean mm_port_probe_run_finish (MMPortProbe *self,
                                    GAsyncResult *result,
                                    GError **error);
-gboolean mm_port_probe_run_cancel (MMPortProbe *self);
 
 gboolean mm_port_probe_run_cancel_at_probing (MMPortProbe *self);
 
@@ -130,6 +130,7 @@ gboolean      mm_port_probe_is_mbim          (MMPortProbe *self);
 const gchar  *mm_port_probe_get_vendor       (MMPortProbe *self);
 const gchar  *mm_port_probe_get_product      (MMPortProbe *self);
 gboolean      mm_port_probe_is_icera         (MMPortProbe *self);
+gboolean      mm_port_probe_is_ignored       (MMPortProbe *self);
 
 /* Additional helpers */
 gboolean mm_port_probe_list_has_at_port   (GList *list);
